#!/bin/env python
# -*- coding: utf-8 -*-

# ------------------------------------------------------------
# opensecurityd
#   
# the opensecurityd as RESTful server
#
# Autor: Mihai Bartha, <mihai.bartha@ait.ac.at>       
#
# Copyright (C) 2013 AIT Austrian Institute of Technology
# AIT Austrian Institute of Technology GmbH
# Donau-City-Strasse 1 | 1220 Vienna | Austria
# http://www.ait.ac.at
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation version 2.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, 
# Boston, MA  02110-1301, USA.
# ------------------------------------------------------------


# ------------------------------------------------------------
# imports

import logging
import os
import urllib
import urllib2

# local
from environment import Environment

class DictDiffer(object):
    """
    Calculate the difference between two dictionaries as:
    (1) items added
    (2) items removed
    (3) keys same in both but changed values
    (4) keys same in both and unchanged values
    """
    def __init__(self, current_dict, past_dict):
        self.current_dict, self.past_dict = current_dict, past_dict
        self.set_current, self.set_past = set(current_dict.keys()), set(past_dict.keys())
        self.intersect = self.set_current.intersection(self.set_past)
    def added(self):
        return self.set_current - self.intersect 
    def removed(self):
        return self.set_past - self.intersect 
    def changed(self):
        return set(o for o in self.intersect if self.past_dict[o] != self.current_dict[o])
    def unchanged(self):
        return set(o for o in self.intersect if self.past_dict[o] == self.current_dict[o])
    
class OpenSecurityException(Exception):
    def __init__(self, value):
        self.value = value
    def __str__(self):
        return repr(self.value)
    
def setupLogger(name='OpenSecurity'):
    logger = logging.getLogger(name)
    logger.setLevel(logging.DEBUG)
    # create formatter and add it to the handlers
    formatter = logging.Formatter('%(asctime)-15s - %(name)s - %(levelname)s - %(message)s')
    # create file handler which logs even debug messages
    fh = logging.FileHandler(os.path.join(Environment('OpenSecurity').log_path, name + '.log'))
    fh.setLevel(logging.DEBUG)
    fh.setFormatter(formatter)
    logger.addHandler(fh)
    # create console handler with a higher log level
    ch = logging.StreamHandler()
    ch.setLevel(logging.DEBUG)
    ch.setFormatter(formatter)
    logger.addHandler(ch)
    return logger

logger = setupLogger()


def showTrayMessage(text, timeout):
    """show a message on the system tray
    
    On windows this is shown like a ballon message.
    
    @param  text        the text to show
    @param  timeout     timeout hint in millisecs for the message 

    """
    try:
        d = { 'text': str(text), 'timeout': int(timeout) }
        urllib2.urlopen('http://127.0.0.1:8090/message?' + urllib.urlencode(d))
    except:
        pass


# test method
def test():

    """Test: OpenSecurity logging"""
    logger.info('test logging')
    showTrayMessage('tray message test\nwith mutliple lines', 1000)


# test the module           
if __name__ == '__main__':
    test()
